@@ -32,6 +32,19 @@ from stestr import utils
 
 
 class Load(command.Command):
+    """Load a subunit stream into a repository.
+
+    Failing tests are shown on the console and a summary of the stream
+    is printed at the end.
+
+    Without --subunit, the process exit code will be non-zero if the test
+    run was not successful. With --subunit, the process exit code is
+    non-zero if the subunit stream could not be generated successfully.
+    The test results and run status are included in the subunit stream, so
+    the stream should be used to determining the result of the run instead
+    of the exit code when using the --subunit flag.
+    """
+
     def get_parser(self, prog_name):
         parser = super(Load, self).get_parser(prog_name)
         parser.add_argument("files", nargs="*", default=False,
@@ -70,14 +83,6 @@ class Load(command.Command):
                             'execution')
         return parser
 
-    def get_description(self):
-        help_str = """Load a subunit stream into a repository.
-
-            Failing tests are shown on the console and a summary of the stream
-            is printed at the end.
-            """
-        return help_str
-
     def take_action(self, parsed_args):
         user_conf = user_config.get_user_config(self.app_args.user_config)
         args = parsed_args
